#!/bin/bash
# 2010 niekt0@hysteria.sk 
# Under GNU/GPL

# This is just simple script for listing "all" structures.
# Take it as example, or modify to satisfy your needs

#XXX change utility paths
foriana=./foriana
bgrep=./bgrep

if [ -z "$1" ]; then
	echo "Usage: $0 m/p dump_file [options to bgrep]";
	echo "Cache file must be available already."
	echo "bgrep parameters: -e <num> how many substitutions can occur";
	echo "                  -s <num> how many first bytes are fixed";
	echo "Example: $0 p freebsd_256M_i386.dd \"-e 10 -s 1\""
	exit;
fi

mode="";

if [ "$1" == "m" ]; then
	mode="module";
	echo "Listing modules."
fi
if [ "$1" == "p" ]; then
	mode="process";
	echo "Listing processes."
fi

if [ "$mode" == "" ]; then
	echo "Invalid mode. Use \"m\" or \"p\"."
fi

if [ ! -f "$2" ]; then
	echo "Dump file does not exists."
	exit;
fi

# $2 = dump file
file=`basename "$2"`;
errcount="$3";

if [ ! -f ".$file" ]; then
	echo "Cache file \".$file\" not found";
	exit;
fi


if [ ! -x "$foriana" ]; then
	echo "executable $foriana not found.";
	exit;
fi

if [ ! -x "$bgrep" ]; then
	echo "executable $bgrep not found.";
	exit;
fi

# Ok, lets do somethink

if [ "$mode" == "process" ]; then

	# get process pattern
	echo -n "Generating process pattern: "
	$foriana --create-process-pattern --list-skip-areas "$2" | \
		grep -A 1 "PROCESS PATTERN START" | \
		tail -1 > ."$file"_process_pattern && \
		echo "OK, saved to \"."$file"_process_pattern\"";
fi

if [ "$mode" == "module" ]; then

	# get module pattern
	echo -n "Generating module pattern: "
	$foriana --create-module-pattern "$2" | \
		grep -A 1 "MODULE PATTERN START" | \
		tail -1 > ."$file"_module_pattern && \
		echo "OK, saved to \"."$file"_module_pattern\"";
fi

# Sometimes helps to ignore NZ bytes, and use then as ??
# cat "$file"_process_pattern | sed 's%/NZ/??/g' > "$file"_process_pattern_...

# optimize pattern => remove leading ?? and NZ and remember their number
# matching ?? at beginning of pattern is CPU expensive

echo -n "Creating optimized pattern: "
cat ."$file"_"$mode"_pattern | sed 's/^[??,NZ]*//' > \
	."$file"_"$mode"_pattern_optimized
len1=`wc -c ."$file"_"$mode"_pattern | cut -d ' ' -f 1`;
len2=`wc -c ."$file"_"$mode"_pattern_optimized | cut -d ' ' -f 1`;
opt_offset=$((($len1 - $len2) / 2));
echo "OK, saved to \"."$file"_"$mode"_pattern_optimized\"";
echo "-----PATTERN START-----"
cat ."$file"_"$mode"_pattern_optimized;
echo "-----PATTERN END-----"

# find all occurrences of pattern
echo -n "Finding all occurrences: "
$bgrep $3 `cat ."$file"_"$mode"_pattern_optimized` "$2" > ."$file"_"$mode"es && echo "OK, saved to \"."$file"_"$mode"es\"";
count=`wc -l ."$file"_"$mode"es | cut -d ' ' -f 1`;

# to see names instantly
# cache file will be changed to XML format
if [ "$mode" == "process" ]; then

	struct_offset=`cat ."$file" | grep '^ts_hl_comm_dst:' | cut -d : -f 2`;
fi
if [ "$mode" == "module" ]; then
	struct_offset=`cat ."$file" | grep '^md_hl_name_dst:' | cut -d : -f 2`;
fi

echo "Using offset: $struct_offset";

echo "Structure addr: String addr: xxd format"
echo "(chars after 0x00 can reveal fork history)";

# print addresses + process names
cat ."$file"_"$mode""es" | cut -d ' ' -f 2 | while read i; do i=$((0x$i-$opt_offset)) && printf "%llx: " "$i" && j=$(($i + $struct_offset)) && xxd -l 16 -s "$j" "$2"; done
echo "Found $count ""$mode""es.";

