#ifndef _ARM_FUNCTIONS_H
#define _ARM_FUNCTIONS_H

/*
 *	Functions for analysing arm architecture systems
*/

#include "../../globals.h"

#include "magic.h" 

/*
 * Initialization function.
 */

int arm_init(struct arch_struct *);

/*
 * Function that guess, if we are really 
 * analysing arm architecture.
 *
 * return 0 on success, otherwise error > 0
 */

int arm_guess();

/*
 * Function that sets pointers to all other 
 * architecture dependent functions and hooks
 *
 */

int arm_arch_settings();


/*
 * Function that manages mapping from linear to physical address space.
 * 
 */
int arm_read_linear(u_int64_t PGD, u_int64_t lin_addr, 
			void *out_data, u_int64_t priv_block_size,
			 u_int64_t *dump_offset );


/*
 * Function for verifying if given virtual address contains VPT 
 * (resp. PaGeDirectory) of some process.
 */
int arm_user_VPT_test(u_int64_t my_addr);

/*
 * Function for verifying if given physical address 
 * contains system VPT (resp. PaGeDirectory)
 *
 */
int arm_VPT_test(u_int64_t my_addr);

/*
 * Function that tries to determine general information 
 * about analyzed image.
 *
 * for now, only system VPT is found.
 * if mode == 1, then all possible VPT candidates are printed.
 *
 * arm_determine ()
 * return 0 on success, otherwise error > 0
 * Discovered information are saved in global structure image1.
 *
 */
int arm_determine(int mode);

#endif
