#ifndef _STRUCTS_H
#define _STRUCTS_H

#include "arch/allarch.h"
#include "os/allos.h"

#define ARCH_NAME_LEN	32
#define OS_NAME_LEN	32


// Typedefs for function.
// We need this, because each architecture have different low-level routines,
// and target architecture is determined on-the-fly

struct arch_struct;
struct os_struct;

typedef  int (*read_linear_t) (u_int64_t PGD,
				u_int64_t lin_addr, 
				void *out_data,
				u_int64_t priv_block_size,
				u_int64_t *dump_offset);
typedef  int (*VPT_test_t) (u_int64_t my_addr);
typedef  int (*user_VPT_test_t) (u_int64_t my_addr);
typedef  int (*find_VPT_t) ();
typedef  int (*arch_init_t) (struct arch_struct *);
typedef  int (*arch_guess_t) ();
typedef  int (*os_init_t) (struct os_struct *);
typedef  int (*os_hook_t) (); // generic hook type, no params
typedef  int (*os_proc_determine_hook_t) (u_int64_t *my_addr);
typedef  int (*os_verify_task_struct_hook_t) (int *hl_size,
						int i,
						u_int64_t ver_addr,
						u_int64_t *bsd_list_start,
						int *found_os,
						int *max_bsd_list_strings_size);
typedef  int (*os_verify_task_struct_hook2_t) (int found_os,
						u_int64_t bsd_list_start,
						u_int64_t *list_start);
typedef  int (*os_hook_list_processes_t) (int *list_areas);

// architecture specific information
struct arch_struct{
	// architecture name (string)
	char architecture[ARCH_NAME_LEN];

	// pointers to architecture specific functions
	read_linear_t read_linear;
	VPT_test_t VPT_test;
	user_VPT_test_t user_VPT_test;
	find_VPT_t find_VPT;
	arch_init_t init;
	arch_guess_t guess;

	// sizes used by architecture
	int pointer_size;
	int integer_size;
};

struct os_struct{
	// os name (string)
	char operating_system[OS_NAME_LEN];

	// name of one of processes for this os
	// this is needed for initial search
	char magic_process_name[TS_COMM_LEN];

	// name of one of user processes for this os
	// this is sometimes needed for initial search
	char magic_user_process_name[TS_COMM_LEN];

	// name of one of processes for this os
	// this is needed for initial search
	char magic_module_name[MD_NAME_LEN];

	// pointer to initialization function
	os_init_t init;

	// pointers to hook functions
	os_proc_determine_hook_t hook_proc_determine;
	os_hook_t hook_proc_determine2;
	os_verify_task_struct_hook_t hook_verify_task_struct;
	os_verify_task_struct_hook2_t hook_verify_task_struct2;
	os_hook_list_processes_t hook_list_processes;

	// if this system is using cyclic list_heads
	int cyclic_lh;
	
	// These parameter cannot be changed from command line

	// Process specific:

	// distance in list_head occurrences between name string
	// and linked list in process structure
	int ts_magic_headlist_number;	

	// How far from TS_MAGIC_HEADLIST_FRONT_NUMBER to look 
	// for right one list_head 
	// (both sides, unit: list_heads;)
	// See theoretical explanation for more information.
	int ts_magic_headlist_spread;
	int ts_magic_headlist_front_number;
	
	// At least how many processes must be in process list
	int ts_magic_min_list_size;

	// How many times try to find list_head inside process structure
	int ts_magic_max_tries;

	// distance from "tasks" list_head to "mm" mmstruct
	// (in linux)
	int ts_magic_hl_mm_dst_p;

	// Maximally how many errors can be in head_list
	int ts_magic_list_max_err;

	// Max. how many list_head we expect before name string 
	// in process structure
	int ts_magic_headlist_max;
	
	// distance from beginning of mm_struct to pgd variable
	// (in linux)
	int mm_magic_pgd_dst_p;

	// Module Specific:

	// distance in list_head occurrences between 
	// list_head and name in module struct :) 
	// actually not used, because they are just near
	// (in linux)
	int md_magic_headlist_number;
	int md_magic_max_tries;
	int md_magic_hl_name_dst_p;
	int md_magic_headlist_max;
	int md_pattern_spread;

	// Maximal size of analyzed list
	int max_list_size;
	// TODO: open files
};

// information about file, we are analyzing
struct my_image{ 

	// size of block in analyzed image
	int guest_block_size;

	// hw architecture of analyzed image
	// This is only string for user
	char architecture[ARCH_NAME_LEN];	

	// architecture of analyzed system. (Index to array)
	int arch;			

	// how many architectures are currently supported
	int archcount;

	// operating system if analyzed image
	// This is only string for user
	char operating_system[OS_NAME_LEN];
	
	// -||-, but number representation (Index to array)
	int os;
	/* 0 - unknown
	 * other may be in various order 
	 */

	// how many operating systems are currently supported
	int oscount;

	// position of VPT (from beginning of file)
	u_int64_t VPT_address;	
	// if true, value of VPT_address is ignored, 
	// and hard coded values are used
	int VPT_not_available;	
	// if true, PGD value was supplied by user, 
	// therefore determination is suppressed 
	int VPT_user_specified;
	// size of image in kilobytes
	u_int64_t ram_size;	
	// distance between comm and task headlist fields in task_struct
	int ts_hl_comm_dst;	  
	// distance between name and modules headlist fields in module
	int md_hl_name_dst;
	// address of task list_head of one of taks_struct structures
	u_int64_t ts_one_of_lists;
	// address of list list_head of one of module structures
	u_int64_t md_one_of_lists;
	// Distance from "tasks" list_head to "mm" mmstruct
	// =TS_MAGIC_HL_MM_DST; 
	int ts_hl_mm_dst; 
	//Distance from beginning of mm_struct to pgd variable
	// =MM_MAGIC_PGD_DST; 
	int mm_pgd_dst;
	// Distance from beginning of vm_area struct to sizes of VMA
	int vm_size_dst;
	// Distance from beginning of vm_area struct to list_head
	int vm_list_dst;

	// how many headl_lists there are in task_struct before comm
	int ts_headlists_number;
	// how many headl_lists there are in module before name
	int md_headlists_number;
	// pointers to headlist of task_struct of one process
	u_int64_t ts_headlists[TS_MAGIC_HEADLIST_MAX];
	// pointers to headlist of module
	u_int64_t md_headlists[MD_MAGIC_HEADLIST_MAX];

	
// stuff for listing of open files:

// => task_struct ->  files   ->    fdt   ->  fd    ->  f_path  ->  mnt
//                (files_struct)  (fdtable)  (file **)   (path)   (vfsmount)
//                                                              ->  dentry -> d_name -> name
//                                                                 (dentry)   (qstr)  (char)

	// Distance from "tasks" list_head to "files" files_struct in task_struct
	int ts_hl_fs_dst;
	// Distance from beginning of structure to "fdt" fdtable in files_struct
	int fs_fdt_dst;
	// Distance from beginning of structure to "fd" **file in fdtable
	int fdt_fd_dst;
	// Distance from beginning of structure to "f_path" path in file structure
	int fl_fpath_dst;
	// Distance from beginning of structure to "dentry" dentry in path structure
	int pt_dentry_dst;
	// Distance from beginning of structure to "d_name" qstr in dentry structure
	int dt_name_dst;
	// Distance from beginning of structure to "name" *char in qstr. 
	// note: Should not change, this structure is elemental
	int qs_name_dst;
	// Distance from beginning of structure to vfsmount in fs_struct.
	int fs_vfs_dst;
	// Pointer to one of path structs
	u_int64_t one_of_paths;		

// This part is added later, I have put it here, to make clear that it is new

	// arch_struct settings

	// Pointer size of analyzed architecture, determined on-the-fly
	int pointer_size;
	// Integer size of analyzed architecture, determined on-the-fly
	int integer_size;

	// Address of {arch}_read_linear function
	read_linear_t read_linear; 
	// Address of {arch}_VPT_test function
	VPT_test_t VPT_test; 
	// Address of {arch}_user_VPT_test function
	user_VPT_test_t user_VPT_test; 
	// Address of {arch}_find_VPT function
	find_VPT_t find_VPT; 

	// os_struct settings
	// if system is using cyclic list_heads, determined on-the-fly
	int cyclic_lh;

	// Parameters from user.
	// user can specify magic constants
	char magic_module[MD_NAME_LEN]; 
	char magic_process[TS_COMM_LEN]; 
	char magic_user_process[TS_COMM_LEN];

	// this structure should contain all OS related variables.
	struct os_struct *os1;

};
#endif

